/*
 *  pH-admin.c
 *
 *  Copyright (C) 2000, 2001  Anil Somayaji
 *
 *  A utility for controlling pH, a kernel extension for online
 *  process intrusion detection monitoring and response.
 *
 *  Licensed under the GNU GPL (see COPYING file), and as such has NO
 *  WARRANTY!  In particular, pH modifies the filesystem using
 *  low-level access routines.  Have a good backup, you could lose
 *  data through disk corruption!
 *
 *  August 16, 2000:    pH-0.1, initial release
 *  July 17, 2001:      pH-0.17
 *
 *  $Id: pH-admin.c,v 1.30 2002/10/18 02:15:11 soma Exp $ */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <asm/unistd.h>
#include <errno.h>

#define PH_STOPMON 0
#define PH_STARTMON 1
#define PH_DELAYFACTOR 2
#define PH_MODMIN 3
#define PH_NORMALMIN 4
#define PH_TOLERIZE_LIMIT 5
#define PH_ANOM_LIMIT 6
#define PH_NORMFACTOR 7
#define PH_RESET_PROFILE 8
#define PH_TOLERIZE 9
#define PH_SENSITIZE 10
#define PH_STATUS 11
#define PH_LOGLEVEL 12
#define PH_LOG_SYSCALLS 13
#define PH_START_NORMAL 14
#define PH_LOG_SEQUENCES 15
#define PH_WRITE_PROFILES 16
#define PH_SUSPEND_EXECVE 17
#define PH_GETMSG 18
#define PH_STARTSIGNAL 19
#define PH_STOPSIGNAL 20
#define PH_FREQ_DELAY 21
#define PH_FREQ_INTERVAL 22
#define PH_NORMAL_WAIT 23
#define PH_SUSPEND_EXECVE_TIME 24
#define PH_RESET_LFC 25

inline int pH(int op, int arg1)
{
        long __res;
        __asm__ volatile ("int $0x80"
                          : "=a" (__res)
                          : "0" (222),"b" ((long)(op)),"c" ((long)(arg1)));

        if ((unsigned long)(__res) >= (unsigned long)(-125)) {
                errno = -(__res);
                __res = -1;
        }

        return (int) (__res);
}


void print_usage(char *filename)
{
        fprintf(stderr,
                "Usage: %s start | stop | startsignal | stopsignal | status |\n"
                "       loglevel <0-3> |\n"
                "       freq <0 or 1> |\n"
                "       log-syscalls <0 or 1> |\n"
                "       log-sequences <0 or 1> |\n"
                "       write-profiles \n"
                "       delay-factor <delay scaling factor> |\n"
                "       normal-min <min # calls in normal profile> |\n"
                "       mod-min <min # calls since profile modified> |\n"
                "       normal-wait <secs to wait to verify normal> |\n"
                "       normal-factor <ratio of normal calls/train calls> |\n"
                "       anomaly-limit <# anomalies allowed before normal "
                "reset> |\n"
                "       suspend-execve <minimum LFC for execve suspends, "
                "0 disables> |\n"
                "       suspend-execve-time <secs for execve suspends> |\n"
                "       reset <pid for profile reset> |\n"
                "       tolerize-limit <maximum LFC to tolerate> |\n"
                "       start-normal <pid for train->test copy> |\n"
                "       clear-lfc <pid for LFC reset> |\n"
                "       tolerize <pid for normal=0> |\n"
                "       sensitize <pid for train reset>\n",
                filename);
}

int main(int argc, char *argv[])
{
        int retval, i;
        
        if ((argc > 3) || (argc < 2)) {
                print_usage(argv[0]);
                return -1;
        }

        for (i = 0; i < strlen(argv[1]); i++) {
                if (argv[1][i] == '_')
                        argv[1][i] = '-';
        }

        retval = 0;

        if (argc == 3) {
                if (strcmp(argv[1], "delay-factor") == 0) {
                        int delay = atoi(argv[2]);
                        retval = pH(PH_DELAYFACTOR, delay);
                        printf("Delay factor changed from %d to %d.\n",
                               retval, delay);
                } else if (strcmp(argv[1], "log-syscalls") == 0) {
                        int log_syscalls = atoi(argv[2]);
                        retval = pH(PH_LOG_SYSCALLS, log_syscalls);
                        if (log_syscalls > 0) {
                                printf("All system call logging started.\n");
                        } else {
                                printf("All system call logging stopped.\n");
                        }
                } else if (strcmp(argv[1], "freq-delay") == 0) {
                        int freq_delay = atoi(argv[2]);
                        retval = pH(PH_FREQ_DELAY, freq_delay);
                        printf("Frequency delay changed from %d to %d.\n",
                               retval, freq_delay);
                } else if (strcmp(argv[1], "freq-interval") == 0) {
                        int freq_interval = atoi(argv[2]);
                        retval = pH(PH_FREQ_INTERVAL, freq_interval);
                        printf("Frequency interval changed from %d to %d.\n",
                               retval, freq_interval);
                } else if (strcmp(argv[1], "log-sequences") == 0) {
                        int log_sequences = atoi(argv[2]);
                        retval = pH(PH_LOG_SEQUENCES, log_sequences);
                        if (log_sequences > 0) {
                                printf("All sequence logging started.\n");
                        } else {
                                printf("All sequence logging stopped.\n");
                        }
                } else if (strcmp(argv[1], "normal-min") == 0) {
                        int normmin = atoi(argv[2]);
                        retval = pH(PH_NORMALMIN, normmin);
                        printf("Normal minimum changed from %d to %d.\n",
                               retval, normmin);
                } else if (strcmp(argv[1], "mod-min") == 0) {
                        int modmin = atoi(argv[2]);
                        retval = pH(PH_MODMIN, modmin);
                        printf("Modified minimum changed from %d to %d.\n",
                               retval, modmin);
                } else if (strcmp(argv[1], "anomaly-limit") == 0) {
                        int anom_limit = atoi(argv[2]);
                        retval = pH(PH_ANOM_LIMIT, anom_limit);
                        printf("Anomaly limit changed from %d to %d.\n",
                               retval, anom_limit);
                } else if ((strcmp(argv[1], "suspend-execve") == 0) ||
                           (strcmp(argv[1], "abort-execve") == 0)){
                        int suspend_execve = atoi(argv[2]);
                        retval = pH(PH_SUSPEND_EXECVE, suspend_execve);
                        printf("minimum LFC for execve suspend (abort) "
                               "changed from %d to %d.\n",
                               retval, suspend_execve);
                } else if ((strcmp(argv[1], "suspend-execve-time") == 0) ||
                           (strcmp(argv[1], "susp-exec-time") == 0)) {
                        int suspend_execve_time = atoi(argv[2]);
                        retval = pH(PH_SUSPEND_EXECVE_TIME,
                                    suspend_execve_time);
                        printf("execve suspend time changed from "
                               " %d to %d seconds.\n",
                               retval, suspend_execve_time);
                } else if (strcmp(argv[1], "tolerize-limit") == 0) {
                        int tol_limit = atoi(argv[2]);
                        retval = pH(PH_TOLERIZE_LIMIT, tol_limit);
                        printf("Tolerization limit changed from %d to %d.\n",
                               retval, tol_limit);
                } else if (strcmp(argv[1], "normal-factor") == 0) {
                        int norm_factor = atoi(argv[2]);
                        retval = pH(PH_NORMFACTOR, norm_factor);
                        printf("Normal ratio factor changed from %d to %d.\n",
                               retval, norm_factor);
                } else if (strcmp(argv[1], "normal-wait") == 0) {
                        int norm_wait = atoi(argv[2]);
                        retval = pH(PH_NORMAL_WAIT, norm_wait);
                        printf("Normal wait time changed from %d to %d.\n",
                               retval, norm_wait);
                } else if (strcmp(argv[1], "loglevel") == 0) {
                        int loglevel = atoi(argv[2]);
                        retval = pH(PH_LOGLEVEL, loglevel);
                        printf("Loglevel changed from %d to %d.\n",
                               retval, loglevel);
                } else if (strcmp(argv[1], "reset") == 0) {
                        int pid = atoi(argv[2]);
                        retval = pH(PH_RESET_PROFILE, pid);
                        if (retval == -1) {
                                retval = errno;
                                if (errno == ESRCH) {
                                        printf("Process %d doesn't exist\n",
                                               pid);
                                        retval = -1;
                                } else {
                                        printf("Process %d isn't being "
                                               "monitored.\n", pid);
                                        retval = -2;
                                }
                        } else {
                                printf("Profile for %d reset.\n", pid);
                                retval = 0;
                        }
                } else if (strcmp(argv[1], "start-normal") == 0) {
                        int pid = atoi(argv[2]);
                        retval = pH(PH_START_NORMAL, pid);
                        if (retval == -1) {
                                retval = errno;
                                if (errno == ESRCH) {
                                        printf("Process %d doesn't exist\n",
                                               pid);
                                        retval = -1;
                                } else {
                                        printf("Process %d isn't being "
                                               "monitored.\n", pid);
                                        retval = -2;
                                }
                        } else {
                                printf("Normal monitoring started in %d "
                                       "(train copied to test).\n", pid);
                                retval = 0;
                        }
                } else if (strcmp(argv[1], "sensitize") == 0) {
                        int pid = atoi(argv[2]);
                        retval = pH(PH_SENSITIZE, pid);
                        if (retval == -1) {
                                retval = errno;
                                if (errno == ESRCH) {
                                        printf("Process %d doesn't exist\n",
                                               pid);
                                        retval = -1;
                                } else {
                                        printf("Process %d isn't being "
                                               "monitored.\n", pid);
                                        retval = -2;
                                }
                        } else {
                                printf("Process %d sensitized "
                                       "(train reset).\n", pid);
                                retval = 0;
                        }
                } else if ((strcmp(argv[1], "reset-lfc") == 0) ||
                           (strcmp(argv[1], "clear-lfc") == 0)) {
                        int pid = atoi(argv[2]);
                        retval = pH(PH_RESET_LFC, pid);
                        if (retval == -1) {
                                retval = errno;
                                if (errno == ESRCH) {
                                        printf("Process %d doesn't exist\n",
                                               pid);
                                        retval = -1;
                                } else {
                                        printf("Clear LFC of %d failed "
                                               "for unknown reason.\n",
                                               pid);
                                        retval = -2;
                                }
                        } else {
                                printf("LFC for process %d cleared (reset).\n",
                                       pid);
                                retval = 0;
                        }
                } else if (strcmp(argv[1], "tolerize") == 0) {
                        int pid = atoi(argv[2]);
                        retval = pH(PH_TOLERIZE, pid);
                        if (retval == -1) {
                                retval = errno;
                                if (errno == ESRCH) {
                                        printf("Process %d doesn't exist\n",
                                               pid);
                                        retval = -1;
                                } else {
                                        printf("Process %d isn't being "
                                               "monitored.\n", pid);
                                        retval = -2;
                                }
                        } else {
                                printf("Process %d tolerized "
                                       "(profile normal=0).\n", pid);
                                retval = 0;
                        }
                } else {
                        print_usage(argv[0]);
                        return -3;
                }
        }

        if (argc == 2) {
                if (strcmp(argv[1], "start") == 0) {
			system("/bin/mkdir -p /var/lib/pH/profiles");
                        retval = pH(PH_STARTMON, 0);
                        printf("Monitoring started.  Call returned %d\n",
                               retval);
                } else if (strcmp(argv[1], "getmsg") == 0) {
                        retval = pH(PH_GETMSG, 0);
                        printf("GETMSG called.\n");
                } else if (strcmp(argv[1], "stop") == 0) {
                        retval = pH(PH_STOPMON, 0);
                        printf("Monitoring stopped.  Call returned %d\n",
                               retval);
                } else if (strcmp(argv[1], "startsignal") == 0) {
                        retval = pH(PH_STARTSIGNAL, 0);
                        printf("Monitoring signals started.  Call returned %d\n",
                               retval);
                } else if (strcmp(argv[1], "stopsignal") == 0) {
                        retval = pH(PH_STOPSIGNAL, 0);
                        printf("Monitoring signals stopped.  Call returned %d\n",
                               retval);
                } else if (strcmp(argv[1], "status") == 0) {
                        retval = pH(PH_STATUS, 0);
                        printf("Status printed in log. Call returned %d\n",
                               retval);
                } else if (strcmp(argv[1], "write-profiles") == 0) {
                        retval = pH(PH_WRITE_PROFILES, 0);
                        if (retval == 0) {
                                printf("Profiles are being written to disk.  "
                                       "Check logs for errors.\n");
                        } else {
                                printf("There was a problem writing the logs "
                                       "to disk.  Is this a pH kernel?\n");
                        }
                } else {
                        print_usage(argv[0]);
                        retval = -2;
                }
        }

        return retval;
}
