/* pH-archive-profiles.c

   archive compress the profiles directory periodically, so it doesn't suck up
   too much disk space.

   $Id: pH-archive-profiles.c,v 1.1 2001/09/10 03:31:13 soma Exp $
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

int sleep_time = 3 * 60 * 60;
char profiledir[] = "/var/lib/pH/profiles";
char archive_prefix[] = "/profiles";

int main(int argc, char *argv[])
{
        struct stat st;
        time_t epoch_time;
        char buf[BUFSIZ], newfile[BUFSIZ], archive_file[BUFSIZ];
        char *archive_dir;
        int dir_len;

        if ((argc < 2) || (argc > 3)) {
                fprintf(stderr,
                        "Usage: %s <archive-dir> [poll-time]\n\n"
                        "This program periodically copies the contents "
                        "of:\n"
                        "   /var/lib/pH/profiles\n"
                        "to:\n"
                        "   <archive-dir>/profiles-<epoch-time>.tar.gz\n"
                        "(The file is compressed using gzip.)\n\n"
                        "poll-time is the number of seconds to wait before "
                        "checking\n"
                        " - defaults to 10800 (3 hours)\n\n",
                        argv[0]);
                exit(-1);
        }

        if (argc == 3) {
                sleep_time = atoi(argv[3]);
        }

        archive_dir = argv[1];
        
        dir_len = strlen(archive_dir);

        if (dir_len > (BUFSIZ - strlen(archive_prefix) - 1)) {
                fprintf(stderr, "Directory is too long.\n");
                exit(-2);
        }

        if (stat(archive_dir, &st)) {
                fprintf(stderr, "Directory %s does not exist.\n",
                        archive_dir);
                exit(-3);
        }

        if (!S_ISDIR(st.st_mode)) {
                fprintf(stderr, "%s is not a directory.\n",
                        archive_dir);
                exit(-4);
        }

        strcpy(archive_file, archive_dir);
        strcpy(archive_file + dir_len, archive_prefix);

        for (;;) {
                if (stat(profiledir, &st) == 0) {
                        epoch_time = time(NULL);
                        sprintf(newfile, "%s-%ld.tar.gz", 
                                archive_file, epoch_time);
                        system("pH-admin write-profiles >& /dev/null");
                        sleep(5);
                        sprintf(buf, "tar -c --gzip -f %s "
                                "-C /var/lib/pH profiles", newfile);
                        system(buf);
                }
                sleep(sleep_time);
        }
        
        return 0;
}
