/* pH-compress-log.c

   compress the all_logfile periodically, so it doesn't suck up
   too much disk space.

   $Id: pH-compress-log.c,v 1.5 2001/08/18 16:27:38 soma Exp $
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

int split_size = 524288000;
int sleep_time = 10 * 60;
char logfile[] = "/var/lib/pH/all_logfile";
char archive_prefix[] = "/all_logfile";

int main(int argc, char *argv[])
{
        struct stat st;
        time_t epoch_time;
        char buf[BUFSIZ], newfile[BUFSIZ], logarchive_file[BUFSIZ];
        char *logarchive_dir;
        int dir_len;

        if ((argc < 2) || (argc > 4)) {
                fprintf(stderr,
                        "Usage: %s <archive-dir> [split-size] [poll-time]\n\n"
                        "This program periodically moves the current contents "
                        "of:\n"
                        "   /var/lib/pH/all_logfile\n"
                        "to:\n"
                        "   <archive-dir>/all_logfile.<epoch-time>.bz2\n"
                        "(The file is compressed using bzip2.)\n\n"
                        "split-size is the minimum number of bytes before a "
                        "split\n"
                        " - defaults to 524288000 (500M)\n"
                        "poll-time is the number of seconds to wait before "
                        "checking\n"
                        " - defaults to 600 (10 minutes)\n\n",
                        argv[0]);
                exit(-1);
        }

        if (argc >= 3) {
                split_size = atoi(argv[2]);
        }

        if (argc == 4) {
                sleep_time = atoi(argv[3]);
        }

        logarchive_dir = argv[1];
        
        dir_len = strlen(logarchive_dir);

        if (dir_len > (BUFSIZ - strlen(archive_prefix) - 1)) {
                fprintf(stderr, "Directory is too long.\n");
                exit(-2);
        }

        if (stat(logarchive_dir, &st)) {
                fprintf(stderr, "Directory %s does not exist.\n",
                        logarchive_dir);
                exit(-3);
        }

        if (!S_ISDIR(st.st_mode)) {
                fprintf(stderr, "%s is not a directory.\n",
                        logarchive_dir);
                exit(-4);
        }

        strcpy(logarchive_file, logarchive_dir);
        strcpy(logarchive_file + dir_len, archive_prefix);

        for (;;) {
                if ((stat(logfile, &st) == 0) && 
                    (st.st_size >= split_size)) {
                        epoch_time = time(NULL);
                        sprintf(newfile, "%s.%ld", logfile, epoch_time);
                        rename(logfile, newfile);
                        system("/usr/local/bin/pH-admin log-syscalls 1");
                        sprintf(buf, "/usr/bin/bzip2 %s", newfile);
                        system(buf);
                        sprintf(buf, "/bin/mv %s.bz2 %s.%ld.bz2",
                                newfile, logarchive_file, epoch_time);
                        system(buf);
                }
                sleep(sleep_time);
        }
        
        return 0;
}
