/*
 *  pH-print-sequences.c
 *
 *  Copyright (C) 2000, 2001  Anil Somayaji
 *
 *  A utility for printing the sequences recorded in a .seq file by
 *  pH.  pH is a kernel extension for online process intrusion
 *  detection monitoring and response.
 *
 *  Licensed under the GNU GPL (see COPYING file), and as such has NO
 *  WARRANTY!  In particular, pH modifies the filesystem using
 *  low-level access routines.  Have a good backup, you could lose
 *  data through disk corruption!
 *
 *  August 16, 2000:    pH-0.1, initial release
 *  July 17, 2000:      pH-0.17
 *
 *  $Id: pH-print-sequences.c,v 1.7 2001/08/07 00:32:37 soma Exp $ */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>

#define PH_SYSCALLS 196       /* note: this value is used as the */
                              /* "no system call" marker in sequences */
#define PH_MAX_SEQLEN 9

typedef unsigned char u8;

typedef struct pH_seq_s {
        int first;                    /* seq is a circular array;        */
                                      /* this is its beginning           */
        int length;
        u8 data[PH_MAX_SEQLEN];   /* current sequence being filled   */
                                      /* or processed                    */
} pH_seq;

typedef struct pH_seq_logrec {
        unsigned long count;
        int pid;
        struct timeval time;
        pH_seq seq;
} pH_seq_logrec;

int main(int argc, char *argv[])
{
        pH_seq_logrec rec;
        pH_seq seq;
        struct tm *tm;
        int retval;
        int fd;
        int file;
        int num_sequences;
        int i, start;
        int graph = 0;

        if (argc < 2) {
                fprintf(stderr, "No arguments!\n");
                exit(-1);
        }

        /* -t for a time graph, -s for a sequence graph */
        if (strcmp(argv[1],"-t") == 0) {
                start = 2;
                graph = 1;
        } else if (strcmp(argv[1],"-s") == 0) {
                start = 2;
                graph = 2;
        } else {
                start = 1;
        }        

        for (file=start; file < argc; file++) {

                fd = open(argv[file], O_RDONLY, 0);

                if (fd < 0) {
                        fprintf(stderr, "Can't open %s\n", argv[file]);
                        continue;
                }
        
                
                printf("File: %s\n", argv[file]);
                num_sequences = 0;
                while ((retval = read(fd, &rec, sizeof(rec))) ==
                       sizeof(rec)) {
                        num_sequences++;
                        seq = rec.seq;
                        tm = localtime(&rec.time.tv_sec);
                        
                        if (graph == 1) {
                                printf("%lu\t%d\n",
                                       rec.time.tv_sec, num_sequences);
                        } else if (graph == 2) {
                                printf("%lu\t%d\n",
                                       rec.count, num_sequences);
                        } else {
                                printf("%d %lu %d:%02d:%02d.%03lu :", rec.pid, 
                                       rec.count, tm->tm_hour, tm->tm_min, 
                                       tm->tm_sec, rec.time.tv_usec/1000);
                                for (i=0; i<seq.length; i++) {
                                        printf(" %d",
                                               seq.data[(seq.first + i) % 
                                                       seq.length]);
                                }
                                printf("\n");
                        }
                }
                if (graph)
                        printf("\n");
                else
                        printf("\n%d sequences total.\n",
                               num_sequences);
        }
        

        return 0;
}
