/* pH-summarize-files.c */

#include <stdio.h>
#include <stdlib.h>

int compare_doubles(const void *x, const void *y)
{
        double *a = (double *) x;
        double *b = (double *) y;

        if (*a < *b)
                return -1;
        else if (*a > *b)
                return 1;
        else
                return 0;
}

FILE *open_ro_file(char *fn)
{
        FILE *file;

        file = fopen(fn, "r");

        if (!file) {
                fprintf(stderr, "Cannot open %s\n", fn);
                exit(-1);
        }
        
        return file;
}

int main(int argc, char *argv[])
{
        int num_files, ptnum, filenum, filenum_base, num_pts;
        double **y, *x, a, b;
        double *avg, *median, *lowquart, *highquart, *min, *max, *total;
        FILE *file;
        char buf[BUFSIZ];

        if (argc < 2)
                return -1;
        
        num_files = argc - 1;
        filenum_base = 1;
        
        file = open_ro_file(argv[filenum_base]);

        num_pts = 0;
        while (fgets(buf, BUFSIZ, file) && 
               (sscanf(buf, "%lf %lf", &a, &b) == 2)) {
                num_pts++;
        }

        fclose(file);
        
        if (num_pts < 1) {
                fprintf(stderr, "No points in first file.\n");
                return -1;
        }
        
        x         = (double *) malloc(sizeof(double) * num_pts);
        avg       = (double *) malloc(sizeof(double) * num_pts);
        median    = (double *) malloc(sizeof(double) * num_pts);
        lowquart  = (double *) malloc(sizeof(double) * num_pts);
        highquart = (double *) malloc(sizeof(double) * num_pts);
        min       = (double *) malloc(sizeof(double) * num_pts);
        max       = (double *) malloc(sizeof(double) * num_pts);
        total     = (double *) malloc(sizeof(double) * num_pts);

        y = (double **) malloc(sizeof(double *) * num_pts);
        for (ptnum = 0; ptnum < num_pts; ptnum++) {
                y[ptnum] = (double *) malloc(sizeof(double) * num_files);
        }

        file = open_ro_file(argv[filenum_base]);
        for (ptnum = 0; ptnum < num_pts; ptnum++) {
                
                if (fgets(buf, BUFSIZ, file) && 
                    (sscanf(buf, "%lf %lf", &a, &b) == 2)) {
                        x[ptnum] = a;
                        y[ptnum][0] = b;
                } else {
                        fprintf(stderr, "Only %d points in %s, "
                                "but wanted %d\n", ptnum,
                                argv[filenum_base], num_pts);
                        exit(-1);
                }
        }
        fclose(file);

        for (filenum = 0; filenum < num_files; filenum++) {
                file = open_ro_file(argv[filenum + filenum_base]);
                for (ptnum = 0; ptnum < num_pts; ptnum++) {
                        if (fgets(buf, BUFSIZ, file) &&
                            (sscanf(buf, "%lf %lf", &a, &b) == 2)) {
                                if (x[ptnum] != a) {
                                        fprintf(stderr,
                                                "Got wrong x value in %s: "
                                                "expected %f, got %f.\n",
                                                argv[filenum + filenum_base],
                                                x[ptnum], a);
                                        exit(-1);
                                }
                                y[ptnum][filenum] = b;
                        } else {
                                fprintf(stderr, "Only %d points in %s, "
                                        "but wanted %d\n", ptnum,
                                        argv[filenum + filenum_base], num_pts);
                                exit(-1);
                        }
                }
                fclose(file);
        }

        for (ptnum = 0; ptnum < num_pts; ptnum++) {
                qsort(y[ptnum], num_files, sizeof(double), 
                      compare_doubles);
                total[ptnum] = 0;
                for (filenum = 0; filenum < num_files; filenum++) {
                        total[ptnum] += y[ptnum][filenum];
                }
        }
        
        for (ptnum = 0; ptnum < num_pts; ptnum++) {
                avg[ptnum] = total[ptnum] / num_files;
                min[ptnum] = y[ptnum][0];
                max[ptnum] = y[ptnum][num_files - 1];
                median[ptnum] = y[ptnum][num_files/2];
                lowquart[ptnum] = y[ptnum][num_files/4];
                highquart[ptnum] = y[ptnum][(num_files * 3)/4];
                
                printf("%f %f %f %f %f %f %f %f\n",
                       x[ptnum], total[ptnum], avg[ptnum], median[ptnum],
                       lowquart[ptnum], highquart[ptnum],
                       min[ptnum], max[ptnum]);
        }

        return 0;
}
